#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include "cdbslib.h"
#include "error.h"
#include "config.h"
#include "alist.h"
#include "uniqname.h"
#include "cdbserror.h"

//=====================================================================
// GETUNIQNAME -- Calculate a new unique name for a reference file
//
// i: ref		reference file descriptor for an open HEADER
// i: reftype		reference file type
// i: filetype		associated file type (image, table, loadfile)
// i: instrument	instrument name
// u: newlist		new names associated with old file names
// o: newname		the unique name
// i: maxch		maximum length of calculated name
//
// History:
// 10/23/02 46235 MSwam  add WFC3, COS
// 12/17/04 52613 MSwam  add WFPC2 support for FITS file names
// 08/14/08 60643 MSwam  add WFPC2 dxy,w4t support
//
//=====================================================================

int getuniqname (reffile *ref, char *reftype, char *filetype, char *instrument,
		 alist *newlist, char *newname, int maxch)
{
    char	dir[SZ_FNAME+1], root[SZ_FNAME+1], ext[SZ_FNAME+1];
    char	newroot[SZ_FNAME+1], irafdir[SZ_FNAME+1], cdbsdir[SZ_FNAME+1];
    int		code, count;

    cdbs_debug("getuniqname: reftype,filetype,instrument = %s %s %s\n",
               reftype,filetype,instrument);

    // If other files with the same root name have already been renamed,
    // give this file the same root name
    //
    if (getlistname (newlist, name_reffile (ref), newname, maxch) == OK)
	return (OK);

    // The cdbs name is the name the file will be known as 
    // when it is copied to the cdbs directory. It is this 
    // name that we want to ensure is unique.
    //
    if (getirafdir (ref, instrument, reftype, 
		    filetype, irafdir, SZ_FNAME) == ERR) {
	newname[0] = EOS;
	return (ERR);

    } else {
	xlate_fname (irafdir, cdbsdir, SZ_FNAME);
    }

    // Different instruments and file types use different naming conventions
    // Call the appropriate naming routine for the instrument and file type
    //
    code = strmatch (instrument, INSTRUMENTS);

    switch (code) {
    case NO_INST:
	cdbs_error (BADINST, "%s", instrument);
	break;

    case ACS:
    case NIC:
    case STIS:
    case COS:
    case WFC3:
        newinstname (reftype, cdbsdir, code, newroot, SZ_FNAME);
	break;

    case SYNPHOT:
        syntabname (ref, reftype, cdbsdir, code, newroot, SZ_FNAME);
	break;

    case WFPC2:
        if (strstr(filetype,"table") || 
            strstr(reftype,"dxy") ) {
          // WFPC2 tables are FITS files and use new name format 
          // WFPC2 dxy image FITS file uses new name format 
          newinstname (reftype, cdbsdir, code, newroot, SZ_FNAME);
        }
        else {
          // WFPC2 images use old name format 
          oldinstname (cdbsdir, code, newroot, SZ_FNAME);
        }
	break;

    default:
          oldinstname (cdbsdir, code, newroot, SZ_FNAME);
    }

    // Check the list for duplicate root names 
    count = 0;
    if (match_alist (newlist, newroot, &count) != NULL)
      return (ERR);

    // Save the root name in the list of root names 
    breakfname (name_reffile (ref), dir, root, ext, SZ_FNAME);
    add_alist (newlist, root, newroot);

    safecopy (newname, dir, maxch);
    safecat (newname, newroot, maxch);
    safecat (newname, ext, maxch);

    return (OK);
}
